from ngsolve import *
from ngsolve.solve import Draw#, Redraw
from netgen.csg import *


# Creating the coil geometry
# Format of endpoints: end_point[i] = [xi1, yi1, zi1, xi2, yi2, zi2, r] 

end_points = []

radius = 6

end_points.append([0, 0, 0, 198.179, 0, 0, radius])
end_points.append([198.179, 0, 0, 198.179 , 280, 0, radius])
end_points.append([198.179, 280, 0, -182.184, 280, 0, radius])
end_points.append([-182.184, 280, 0, -182.184, 20, 0, radius])
end_points.append([-182.184, 20, 0, 168.162, 20, 0, radius])
end_points.append([168.162, 20, 0, 168.162, 260, 0, radius])
end_points.append([168.162, 260, 0, -152.166, 260, 0, radius])


# Create geometry
geo = CSGeometry()

for i, end_point in enumerate(end_points):
    dir_vec = Vec(end_point[3]-end_point[0],
                  end_point[4]-end_point[1],
                  end_point[5]-end_point[2])
    dir_vec = 1/dir_vec.Norm()*dir_vec
    if(i==0):
        plane_start = Plane(Pnt(end_point[0],end_point[1],end_point[2]),dir_vec)
        print(plane_start)
    else:
        plane_start = plane_end
    if(i<len(end_points)-1):
        next_dir_vec = Vec(end_points[i+1][3]-end_points[i+1][0],
                           end_points[i+1][4]-end_points[i+1][1],
                           end_points[i+1][5]-end_points[i+1][2])
        next_dir_vec = 1/next_dir_vec.Norm()*next_dir_vec
        end_vec = dir_vec+next_dir_vec
        plane_end = Plane(Pnt(end_point[3],end_point[4],end_point[5]),end_vec)
        print(plane_end)
    else:
        plane_end = Plane(Pnt(end_point[3],end_point[4],end_point[5]),dir_vec)
    
    cyl = Cylinder(Pnt(end_point[0],end_point[1],end_point[2]),
                   Pnt(end_point[3],end_point[4],end_point[5]),
                   end_point[6])*plane_end-plane_start
    print(cyl)
    geo.Add(cyl)

# Draw Geometry
geo.Draw()

# Generate Mesh
mesh = geo.GenerateMesh(maxh=3)
